@@ -54,6 +54,7 @@ sub rightmostvisiblefit {
     my $wremain;
     my $realw;
     my $visualc;
+    my $go;
 
     #   find leftmost column for the rightmost visible fit
     $visualc = 0;
@@ -61,15 +62,23 @@ sub rightmostvisiblefit {
     $wremain = $matrixwidget->{"Width"};
     #printf STDERR "rightmostvisiblefit lastc=%d\n", (defined $lastc) ? $lastc : "undef";
     #printf STDERR "rightmostvisiblefit wremain=%d\n", (defined $wremain) ? $wremain : "undef";
-    for ($c = -1; $c >= -1; $c = $c == -1 ? $lastc: --$c) {
+    $go = 1;
+    $c = -1;
+    do {
         if (not $matrixdata->{"CH.$c"}->{"Colhide"}) {
             my $w = $matrixdata->{"CH.$c"}->{"Width"};
             $realw = $w + $matrixdata->{"CH.$c"}->{"Colgap"};
             $wremain -= $realw;
             $visualc++ if ($c >= 0 and $wremain >= 0);
         }
-        last if ($wremain < 0);
-    }
+        if ($wremain < 0) {
+            $go = 0;
+        }
+        else {
+            $c = $c == -1 ? $lastc : --$c;
+            $go = 0 if ($c < 0);
+        }
+    } while ($go);
     $c++;
     #printf STDERR "rightmostvisiblefit c=%d, visualc=%d\n", $c, $visualc;
     return ($c, $visualc);
@@ -81,6 +90,7 @@ sub bottommostvisiblefit {
     my $hremain;
     my $realh;
     my $visualr;
+    my $go;
 
     #   find topmost column for the bottommost visible fit
     $visualr = 0;
@@ -88,15 +98,24 @@ sub bottommostvisiblefit {
     $hremain = $matrixwidget->{"Height"};
     #printf STDERR "bottommostvisiblefit lastr=%d\n", (defined $lastr) ? $lastr : "undef";
     #printf STDERR "bottommostvisiblefit hremain=%d\n", (defined $hremain) ? $hremain : "undef";
-    for ($r = -1; $r >= -1; $r = $r == -1 ? $lastr: --$r) {
+    $go = 1;
+    $r = -1;
+    do {
         if (not $matrixdata->{"RH.$r"}->{"Rowhide"}) {
             my $h = $matrixdata->{"RH.$r"}->{"Height"};
             $realh = $h + $matrixdata->{"RH.$r"}->{"Rowgap"};
             $hremain -= $realh;
             $visualr++ if ($r >= 0 and $hremain >= 0);
         }
-        last if ($hremain < 0);
-    }
+        #printf STDERR "r=%2d, realh=%2d, hremain=%2d, visualr=%2d\n", $r, $realh, $hremain, $visualr;
+        if ($hremain < 0) {
+            $go = 0;
+        }
+        else {
+            $r = $r == -1 ? $lastr : --$r;
+            $go = 0 if ($r < 0);
+        }
+    } while ($go);
     $r++;
     #printf STDERR "bottommostvisiblefit r=%d, visualr=%d\n", $r, $visualr;
     return ($r, $visualr);
@@ -673,6 +692,62 @@ sub keybinder {
             #$matrixwidget->{"Cursorrow"} = 0;
             return;
         }
+        if ($key eq "s") {
+            {
+                my $wstat = $widget->{'stat'}->width;
+                my $textl = "Accounting System (AS) - save data and continue editing";
+                my $textr = "F1 Help";
+                my $textf = " " x ($wstat -2 -length($textl) -1 -1 -length($textr) - 2) . " ";
+                my $text  = sprintf("  %s %s %s ", $textl, $textf, $textr);
+                $widget->{'info'}->{-text} = $text;
+                $widget->{'info'}->draw;
+            }
+            $breakout = 1;
+            return;
+        }
+        elsif ($key eq "q") {
+            if ($cui->dialog(-message => " Quit without saving? \n" .
+                                         " Changes will be lost ",
+                                           -buttons => 
+                                           [
+                                                { -label => "[ OK ]",     -value => 1 },
+                                                { -label => "[ Cancel ]", -value => 0 }
+                                           ]
+                                            )
+                ) {
+                $widget->{"dummy"}->focus;
+                $breakout = 2;
+            }
+            return;
+        }
+        elsif ($key eq "x" or $key eq "Z") {
+            $breakout = 3;
+            return;
+        }
+        elsif ($key eq KEY_F(1)) {
+            my $text = `man as-cui 2>/dev/null` || "Cannot launch man or man page for as-cui not found";
+            my $w = $this->parent->add("help", 'TextViewer',
+                -x          => $matrixwidget->{"XOffset"},
+                -y          => $matrixwidget->{"YOffset"},
+                -width      => $matrixwidget->{"Width"},
+                -height     => $matrixwidget->{"Height"},
+                -border     => 1,
+                -title      => "Help",
+                -vscrollbar => 1,
+                -text       => $text
+            );
+            $w->set_binding(
+                sub {
+                    my $this = shift;
+                    my $key = shift;
+                    $this->{-has_modal_focus} = 0;
+                },
+                KEY_ENTER(), CUI_ESCAPE());
+            $w->draw;
+            $w->modalfocus;
+            $this->parent->delete("help");
+            $datacoll = 1;
+        }
         return 'DELEGATE';
     }
 
@@ -1310,7 +1385,7 @@ sub ascuinew {
     ];
     my $menu = [
         { -label => 'File',               -submenu => $menu_file         },
-        { -label => 'Accounting',         -submenu => $menu_as           },
+        # FIXME { -label => 'Accounting',         -submenu => $menu_as           },
     ];
     $widget->{'menu'} = $cui->add('menu', 'Menubar',
         -menu => $menu
